import io
import os
import sys
import unittest

import pytest

from domore.cli import main, main_har2case_alias, main_dorun_alias


class TestCli(unittest.TestCase):
    def setUp(self):
        self.captured_output = io.StringIO()
        sys.stdout = self.captured_output

    def tearDown(self):
        sys.stdout = sys.__stdout__  # Reset redirect.

    def test_show_version(self):
        sys.argv = ["hrun", "-V"]

        with self.assertRaises(SystemExit) as cm:
            main()

        self.assertEqual(cm.exception.code, 0)

        from domore import __version__

        self.assertIn(__version__, self.captured_output.getvalue().strip())

    def test_show_help(self):
        sys.argv = ["hrun", "-h"]

        with self.assertRaises(SystemExit) as cm:
            main()

        self.assertEqual(cm.exception.code, 0)

        from domore import __description__

        self.assertIn(__description__, self.captured_output.getvalue().strip())

    def test_debug_pytest(self):
        cwd = os.getcwd()
        try:
            os.chdir(cwd)
            print(os.getcwd())
            exit_code = pytest.main(
                ["-s", "data/trialos.test.com.py", ]
            )
            self.assertEqual(exit_code, 0)
        finally:
            os.chdir(cwd)

    def test_run_testcase_with_raw_har(self):
        sys.argv = ["dorun", "--profile", "profile_test.json", "trialos.test.com.har"]
        cwd = os.getcwd()

        try:
            # os.chdir(os.path.join(os.path.dirname(cwd), "examples", "postman_echo/request_methods/"))
            os.chdir(os.path.join(cwd, "data"))
            with self.assertRaises(SystemExit) as cm:
                main_dorun_alias()
                self.assertEqual(cm.exception.code, 0)
        finally:
            # os.remove(os.path.join(os.getcwd(), sys.argv[-1]))
            os.chdir(cwd)

    def test_gen_testcase(self):
        # Set command line arguments
        sys.argv = ["har2case", "-2j", "trialos.test.com.har"]
        # Get current working directory
        cwd = os.getcwd()
        try:
            os.chdir(os.path.join(cwd, "data"))
            # Run main_har2case_alias function and assert that it exits successfully
            with self.assertRaises(SystemExit) as cm:
                main_har2case_alias()
                self.assertEqual(cm.exception.code, 0)
        finally:
            # Change back to the original working directory
            os.chdir(cwd)

    def test_domo(self):
        sys.argv = ["domo", "-h"]

        with self.assertRaises(SystemExit) as cm:
            main()

        self.assertEqual(cm.exception.code, 0)

        from domore import __description__

        self.assertIn(__description__, self.captured_output.getvalue().strip())
